/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.    
 */
package org.apache.tuscany.sca.policy;

import java.util.List;

/**
 * Base interface for all assembly model objects that can be have policy intents
 * attached to them.
 *
 * @version $Rev: 656636 $ $Date: 2008-05-15 13:50:10 +0100 (Thu, 15 May 2008) $
 */
public interface IntentAttachPoint {

    /**
     * Returns a list of policy intents. See the Policy Framework specification
     * for a description of this attribute.
     * 
     * @return a list of policy intents.
     */
    List<Intent> getRequiredIntents();
    
   
    /**
     * Returns the type of the attach point such as a BindingType or an ImplementationType and so on
     * @return
     */
    IntentAttachPointType getType();
    
    /**
     * 
     * Sets the type of the attach point such as a BindingType or an ImplementationType and so on
     * @param type
     */
    void setType(IntentAttachPointType type);
}
